from glob import glob
from itertools import chain
import re
from os.path import abspath, isdir
from os import rename
import sys

DEFAULTPKGNAME = "yourpackagename"


def do_replacements(
    text_replacements,
    directory_replacements,
    file_extensions,
    additional_files,
    ignore_files,
):

    path_iterator = chain(
        *[glob("**/*." + ext, recursive=True) for ext in file_extensions]
    )
    path_iterator = chain(path_iterator, additional_files)

    messages = []

    path_to_newlines = dict()
    dirs_to_rename = dict()

    for path in path_iterator:
        if abspath(path) in ignore_files:
            continue
        with open(path, "r") as f:
            lines = f.readlines()

        newlines = []
        for x1, x2 in text_replacements:

            pattern = re.compile(x1)
            for linenum, line in enumerate(lines):
                for m in pattern.finditer(line):
                    messages.append(
                        "{} {}:{} | to be replace '{}' with '{}'".format(
                            path, linenum, m.start(), x1, x2
                        )
                    )
                newlines.append(re.sub(x1, x2, line))
        path_to_newlines[abspath(path)] = newlines

    for x1, x2 in directory_replacements:
        if isdir(abspath(x1)):
            messages.append(
                "rename directory\t'{}'\n\t\tto\t'{}'".format(abspath(x1), abspath(x2))
            )
            dirs_to_rename[abspath(x1)] = abspath(x2)

    for msg in messages:
        print(msg)

    print()
    if any(path_to_newlines.values()):
        response = input("Rewrite files? (y|[n]): ")
        if response.lower() == "y":
            print("Rewriting files")
            for path, lines in path_to_newlines.items():
                with open(path, "w") as f:
                    f.writelines(lines)
            print("Renaming directories")
            for oldpath, newpath in dirs_to_rename.items():
                rename(oldpath, newpath)
            return True
        else:
            print("Canceled. No files written.")
            return False
    else:
        print("No replacements found.")
        return False


def initialize(pkgname, fromname=DEFAULTPKGNAME):
    text_replacements = [(fromname, pkgname)]
    file_extensions = ["py", "toml"]
    additional_files = ["Makefile"]
    ignore_files = [abspath(__file__)]
    if do_replacements(
        text_replacements,
        text_replacements,
        file_extensions,
        additional_files,
        ignore_files,
    ):
        print(
            "🐝 Don't forget to edit your package information in the pyproject.toml file!"
        )


if __name__ == "__main__":
    try:
        initialize(sys.argv[1], sys.argv[2])
    except:
        initialize(sys.argv[1])



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['keat_scripts', 'keats']

package_data = \
{'': ['*']}

install_requires = \
['fire>=0.1.3,<0.2.0', 'termcolor>=1.1,<2.0', 'toml>=0.10.0,<0.11.0']

entry_points = \
{'console_scripts': ['keats = keats:main']}

setup_kwargs = {
    'name': 'keats',
    'version': '0.2.11',
    'description': 'Utilities for managing version, changelogs, and project releases.',
    'long_description': '# Keats\n\n![John Keats](assets/keats.jpg)\n\nKeats is an Python build, installation, and workflow manager.\n\nFeatures include forced change logs, release scripts, and package version control.\n\n## Why\n\nEvery Python developer seems to have their own tricks and scripts for\nmaintaining changelogs, package versions, and managing releases. Rather\nthan reinventing the wheel everytime you develop a new pacakge, Keats\nprovides a standard workflow for doing python package releases. No\ncustomization or Makefiles are required.\n\n## Usage\n\nTo install **Keats** to your project, run:\n\n```bash\npoetry run add --dev keats\n```\n\nVerify your installation by running\n\n```bash\npoetry run keats keats\n```\n\nTo list documentation, run keats with no arguments\n\n```bash\npoetry run keats\n```\n\nTo begin version managment by creating a `__version__.py` in\nyour main package, run\n\n```bash\npoetry run keats version up\n```\n\nFrom within your python project, you can access your version number and\nother package information using something like (usually in the `__init__.py`\nof your main package).\n\n```python\nfrom .__version__.py import __version__, __title__, __authors__ # and so on\n```\n\nTo bump to the next version and update your change log (for more on change logs, see below)\n\n```\npoetry run keats bump\n```\n\nTo bump to a specific version:\n\n```\npoetry run keats bump <optional version>\n```\n\nTo bump without updating the change log:\n\n```bash\npoetry run keats version bump\n```\n\n\n**Changelogs**\n\nChangelogs are important understanding project status and developer intentions. \nKeats encourages an up-to-date changelog by providing an standard interface\nfor maintaining and updating change logs using the following files:\n\n* `.keats/changelog.json` - JSON formatted list of changes, with version number, dates, and optional change list.\n* `.keats/changelog.md` - markdown formatted changelog\n\nThe recommended way to use this is to run `keats bump` which will\nbump your package version *and* update your change log:\n\n```\npoetry run keats bump <optional version>\n```\n\nThis will provide an interactive script to update your changelog\nwith a description and a list of changes. Entries are appended to the\n`.keats/changelog.json` and saved. The file is then converted to a markdown\nfile for readability or documentation purposes.\n\nTo just update your change log:\n\n```bash\npoetry run keats changelog add\n```\n\nIf you want to just update the `.keats/changelog.md` from the json file,\nrun:\n\n```bash\npoetry run keats changelog up\n```\n\nTo clear your change logs:\n\n```bash\npoetry rubn keats changelog clear\n```\n\n## Global installation\n\nTo install **Keats** globally, run:\n\n```bash\npip install keats\n```\n\nYou can then run all of the commands without the `poetry run`\nprefix, given that your current directory is a Python project\nwith a `pyproject.toml` file.',
    'author': 'Justin Vrana',
    'author_email': 'justin.vrana@gmail.com',
    'url': 'https://github.com/jvrana/keats',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)
